@@ -12,7 +12,10 @@ from webargs import fields
 from webargs.flaskparser import use_args
 
 from relay.api import fields as custom_fields
-from relay.blockchain.currency_network_proxy import CurrencyNetworkProxy
+from relay.blockchain.currency_network_events import (
+    all_event_types as all_currency_network_event_types,
+    trustline_event_types,
+)
 from relay.blockchain.delegate import (
     IdentityDeploymentFailedException,
     InvalidChainId,
@@ -24,13 +27,12 @@ from relay.blockchain.delegate import (
     InvalidTimeLimit,
     UnknownIdentityFactoryException,
 )
-from relay.blockchain.events_informations import (
+from relay.blockchain.unw_eth_events import all_event_types as all_unw_eth_event_types
+from relay.ethindex_db.events_informations import (
     EventNotFoundException,
     IdentifiedNotPartOfTransferException,
     TransferNotFoundException,
 )
-from relay.blockchain.unw_eth_proxy import UnwEthProxy
-from relay.concurrency_utils import TimeoutException
 from relay.network_graph.payment_path import FeePayer, PaymentPath
 from relay.relay import TrustlinesRelay
 from relay.utils import get_version, sha3
@@ -39,6 +41,7 @@ from .schemas import (
     AccruedInterestListSchema,
     AggregatedAccountSummarySchema,
     AnyEventSchema,
+    AppliedDelegationFeeSchema,
     CurrencyNetworkEventSchema,
     CurrencyNetworkSchema,
     IdentityInfosSchema,
@@ -46,6 +49,7 @@ from .schemas import (
     MetaTransactionSchema,
     MetaTransactionStatusSchema,
     PaymentPathSchema,
+    TransactionIdentifierSchema,
     TransactionStatusSchema,
     TransferIdentifierSchema,
     TransferInformationSchema,
@@ -57,9 +61,6 @@ from .schemas import (
 logger = logging.getLogger("api.resources")
 
 
-TIMEOUT_MESSAGE = "The server could not handle the request in time"
-
-
 def abort_if_unknown_network(trustlines, network_address):
     if not trustlines.is_currency_network(network_address):
         abort(404, "Unknown network: {}".format(network_address))
@@ -93,15 +94,15 @@ def handle_meta_transaction_exceptions(function_to_call):
         except InvalidDelegationFeesException:
             abort(
                 400,
-                f"Invalid delegation fees: fees too low, not supported currency network of fees, "
-                f"or invalid fee recipient",
+                "Invalid delegation fees: fees too low, not supported currency network of fees, "
+                "or invalid fee recipient",
             )
         except InvalidTimeLimit:
             abort(400, f"Invalid time limit for meta-tx: {meta_transaction.time_limit}")
         except identity.ValidateTimeLimitNotFound:
             abort(
                 400,
-                f"The function to validate the time limit was not found in the contract",
+                "The function to validate the time limit was not found in the contract",
             )
         except InvalidChainId:
             abort(400, f"Invalid chain id for meta-tx: {meta_transaction.chain_id}")
@@ -113,17 +114,17 @@ def handle_meta_transaction_exceptions(function_to_call):
         except identity.ValidateNonceNotFound:
             abort(
                 400,
-                f"The function to validate the nonce and hash was not found in the contract",
+                "The function to validate the nonce and hash was not found in the contract",
             )
         except InvalidSignature:
             abort(400, f"Invalid signature for meta-tx: {meta_transaction.signature}")
         except identity.ValidateSignatureNotFound:
             abort(
                 400,
-                f"The function to validate the signature was not found in the contract",
+                "The function to validate the signature was not found in the contract",
             )
         except InvalidMetaTransactionException:
-            abort(400, f"The meta-transaction is invalid")
+            abort(400, "The meta-transaction is invalid")
         except InvalidIdentityContractException as exception:
             abort(
                 400,
@@ -203,7 +204,6 @@ def _get_extended_account_summary(
     account_summary = graph.get_account_sum(a_address, b_address, timestamp=timestamp)
     account_summary.user = a_address
     account_summary.counterParty = b_address
-    account_summary.address = b_address
     account_summary.id = _id(network_address, a_address, b_address)
     account_summary.currencyNetwork = network_address
     return account_summary
@@ -322,7 +322,7 @@ class UserEventsNetwork(Resource):
         "fromBlock": fields.Int(required=False, missing=0),
         "type": fields.Str(
             required=False,
-            validate=validate.OneOf(CurrencyNetworkProxy.event_types),
+            validate=validate.OneOf(all_currency_network_event_types),
             missing=None,
         ),
     }
@@ -333,18 +333,10 @@ class UserEventsNetwork(Resource):
         abort_if_unknown_network(self.trustlines, network_address)
         from_block = args["fromBlock"]
         type = args["type"]
-        try:
-            return self.trustlines.get_user_network_events(
-                network_address, user_address, type=type, from_block=from_block
-            )
-        except TimeoutException:
-            logger.warning(
-                "User network events: event_name=%s user_address=%s from_block=%s. could not get events in time",
-                type,
-                user_address,
-                from_block,
-            )
-            abort(504, TIMEOUT_MESSAGE)
+
+        return self.trustlines.get_user_network_events(
+            network_address, user_address, type=type, from_block=from_block
+        )
 
 
 class TrustlineEvents(Resource):
@@ -355,7 +347,7 @@ class TrustlineEvents(Resource):
         "fromBlock": fields.Int(required=False, missing=0),
         "type": fields.Str(
             required=False,
-            validate=validate.OneOf(CurrencyNetworkProxy.trustline_event_types),
+            validate=validate.OneOf(trustline_event_types),
             missing=None,
         ),
     }
@@ -368,24 +360,14 @@ class TrustlineEvents(Resource):
         abort_if_unknown_network(self.trustlines, network_address)
         from_block = args["fromBlock"]
         type = args["type"]
-        try:
-            return self.trustlines.get_trustline_events(
-                network_address,
-                user_address,
-                counter_party_address,
-                type=type,
-                from_block=from_block,
-            )
-        except TimeoutException:
-            logger.warning(
-                "Trustline events: event_name=%s user_address=%s "
-                "counter_party_address=%s, from_block=%s. could not get events in time",
-                type,
-                user_address,
-                counter_party_address,
-                from_block,
-            )
-            abort(504, TIMEOUT_MESSAGE)
+
+        return self.trustlines.get_trustline_events(
+            network_address,
+            user_address,
+            counter_party_address,
+            type=type,
+            from_block=from_block,
+        )
 
 
 class UserEvents(Resource):
@@ -397,7 +379,7 @@ class UserEvents(Resource):
         "type": fields.Str(
             required=False,
             validate=validate.OneOf(
-                CurrencyNetworkProxy.event_types + UnwEthProxy.event_types
+                all_currency_network_event_types + all_unw_eth_event_types
             ),
             missing=None,
         ),
@@ -408,21 +390,10 @@ class UserEvents(Resource):
     def get(self, args, user_address: str):
         type = args["type"]
         from_block = args["fromBlock"]
-        try:
-            return self.trustlines.get_user_events(
-                user_address,
-                type=type,
-                from_block=from_block,
-                timeout=self.trustlines.event_query_timeout,
-            )
-        except TimeoutException:
-            logger.warning(
-                "User events: event_name=%s user_address=%s from_block=%s. could not get events in time",
-                type,
-                user_address,
-                from_block,
-            )
-            abort(504, TIMEOUT_MESSAGE)
+
+        return self.trustlines.get_user_events(
+            user_address, type=type, from_block=from_block,
+        )
 
 
 class EventsNetwork(Resource):
@@ -433,7 +404,7 @@ class EventsNetwork(Resource):
         "fromBlock": fields.Int(required=False, missing=0),
         "type": fields.Str(
             required=False,
-            validate=validate.OneOf(CurrencyNetworkProxy.event_types),
+            validate=validate.OneOf(all_currency_network_event_types),
             missing=None,
         ),
     }
@@ -444,17 +415,10 @@ class EventsNetwork(Resource):
         abort_if_unknown_network(self.trustlines, network_address)
         from_block = args["fromBlock"]
         type = args["type"]
-        try:
-            return self.trustlines.get_network_events(
-                network_address, type=type, from_block=from_block
-            )
-        except TimeoutException:
-            logger.warning(
-                "Network events: event_name=%s from_block=%s. could not get events in time",
-                type,
-                from_block,
-            )
-            abort(504, TIMEOUT_MESSAGE)
+
+        return self.trustlines.get_network_events(
+            network_address, type=type, from_block=from_block
+        )
 
 
 class UserAccruedInterestList(Resource):
@@ -557,6 +521,18 @@ class TransferInformation(Resource):
             raise RuntimeError("Unhandled input parameters.")
 
 
+class AppliedDelegationFees(Resource):
+    def __init__(self, trustlines: TrustlinesRelay) -> None:
+        self.trustlines = trustlines
+
+    @use_args(TransactionIdentifierSchema())
+    @dump_result_with_schema(AppliedDelegationFeeSchema(many=True))
+    def get(self, args):
+        transaction_hash = args["transactionHash"]
+
+        return self.trustlines.get_paid_delegation_fees_for_tx_hash(transaction_hash)
+
+
 class TransactionInfos(Resource):
     def __init__(self, trustlines: TrustlinesRelay) -> None:
         self.trustlines = trustlines
@@ -696,7 +672,10 @@ class DeployIdentity(Resource):
             )
         except IdentityDeploymentFailedException:
             abort(400, "The identity deployment failed, identity already deployed?")
-        return self.trustlines.get_identity_info(identity_contract_address)
+        try:
+            return self.trustlines.get_identity_info(identity_contract_address)
+        except InvalidIdentityContractException:
+            abort(400, "The deployed identity is invalid.")
 
 
 class IdentityInfos(Resource):
