@@ -1,27 +1,25 @@
-import functools
-import itertools
 import json
 import logging
 import os
-import sys
 import time
 from collections import defaultdict
 from copy import deepcopy
-from typing import Dict, Iterable, List, NamedTuple, Optional, Union
+from typing import Dict, Iterable, List, NamedTuple, Optional
 
 import eth_account
 import eth_keyfile
-import gevent
 import sqlalchemy
 from eth_utils import is_checksum_address, to_checksum_address
-from gevent import sleep
 from sqlalchemy.engine.url import URL
+from tlbin import load_packaged_contracts
 from tldeploy.identity import MetaTransaction
 from web3 import Web3
 
-import relay.concurrency_utils as concurrency_utils
-from relay import ethindex_db, signing_middleware
+from relay import signing_middleware
+from relay.blockchain.identity_events import FeePaymentEventType
+from relay.blockchain.identity_proxy import IdentityProxy
 from relay.blockchain.proxy import LogFilterListener
+from relay.ethindex_db import ethindex_db
 from relay.pushservice.client import PushNotificationClient
 from relay.pushservice.client_token_db import (
     ClientTokenAlreadyExistsException,
@@ -30,6 +28,7 @@ from relay.pushservice.client_token_db import (
 from relay.pushservice.pushservice import (
     FirebaseRawPushService,
     InvalidClientTokenException,
+    create_firebase_app_from_path_to_keyfile,
 )
 from relay.web3provider import create_provider_from_config
 
@@ -42,12 +41,11 @@ from .blockchain import (
 from .blockchain.currency_network_proxy import CurrencyNetworkProxy
 from .blockchain.delegate import Delegate, DelegationFees
 from .blockchain.events import BlockchainEvent
-from .blockchain.events_informations import EventsInformationFetcher
 from .blockchain.exchange_proxy import ExchangeProxy
 from .blockchain.node import Node
-from .blockchain.proxy import Proxy, sorted_events
 from .blockchain.token_proxy import TokenProxy
 from .blockchain.unw_eth_proxy import UnwEthProxy
+from .ethindex_db.events_informations import EventsInformationFetcher
 from .events import BalanceEvent, NetworkBalanceEvent
 from .exchange.orderbook import OrderBookGreenlet
 from .network_graph.graph import CurrencyNetworkGraph
@@ -154,92 +152,55 @@ class TrustlinesRelay:
     def enable_deploy_identity(self) -> bool:
         return self.config["delegate"]["enable_deploy_identity"]
 
-    @property
-    def event_query_timeout(self) -> int:
-        return self.config["trustline_index"]["event_query_timeout"]
-
-    @property
-    def use_eth_index(self) -> bool:
-        return os.environ.get("ETHINDEX", "1") == "1"
-
-    def get_event_selector_for_currency_network(
-        self, network_address: str
-    ) -> Union[ethindex_db.EthindexDB, CurrencyNetworkProxy]:
-        """return either a CurrencyNetworkProxy or a EthindexDB instance
+    def get_ethindex_db_for_currency_network(
+        self, network_address: Optional[str] = None
+    ) -> ethindex_db.CurrencyNetworkEthindexDB:
+        """return an EthindexDB instance.
         This is being used from relay.api to query for events.
         """
-        if self.use_eth_index:
-            return ethindex_db.EthindexDB(
-                ethindex_db.connect(""),
-                address=network_address,
-                standard_event_types=currency_network_events.standard_event_types,
-                event_builders=currency_network_events.event_builders,
-                from_to_types=currency_network_events.from_to_types,
-            )
-        else:
-            return self.currency_network_proxies[network_address]
+        return ethindex_db.CurrencyNetworkEthindexDB(
+            ethindex_db.connect(""),
+            address=network_address,
+            standard_event_types=currency_network_events.standard_event_types,
+            event_builders=currency_network_events.event_builders,
+            from_to_types=currency_network_events.from_to_types,
+        )
 
-    def get_generic_event_selector_for_currency_networks(
-        self,
-    ) -> Union[ethindex_db.EthindexDB, Proxy]:
-        """return either a Proxy or a EthindexDB instance
-        This proxy/db is not bind to a specify currency network address
+    def get_ethindex_db_for_token(self, address: str):
+        """return an EthindexDB instance
         This is being used from relay.api to query for events.
         """
-        if self.use_eth_index:
-            return ethindex_db.EthindexDB(
-                ethindex_db.connect(""),
-                standard_event_types=currency_network_events.standard_event_types,
-                event_builders=currency_network_events.event_builders,
-                from_to_types=currency_network_events.from_to_types,
-            )
-        else:
-            return Proxy(self._web3, abi=self.contracts["CurrencyNetwork"]["abi"])
+        return ethindex_db.EthindexDB(
+            ethindex_db.connect(""),
+            address=address,
+            standard_event_types=token_events.standard_event_types,
+            event_builders=token_events.event_builders,
+            from_to_types=token_events.from_to_types,
+        )
 
-    def get_event_selector_for_token(self, address):
-        """return either a proxy or a EthindexDB instance
+    def get_ethindex_db_for_unw_eth(self, address: str):
+        """return an EthindexDB instance
         This is being used from relay.api to query for events.
         """
-        if self.use_eth_index:
-            return ethindex_db.EthindexDB(
-                ethindex_db.connect(""),
-                address=address,
-                standard_event_types=token_events.standard_event_types,
-                event_builders=token_events.event_builders,
-                from_to_types=token_events.from_to_types,
-            )
-        else:
-            return self.token_proxies[address]
+        return ethindex_db.EthindexDB(
+            ethindex_db.connect(""),
+            address=address,
+            standard_event_types=unw_eth_events.standard_event_types,
+            event_builders=unw_eth_events.event_builders,
+            from_to_types=unw_eth_events.from_to_types,
+        )
 
-    def get_event_selector_for_unw_eth(self, address):
-        """return either a proxy or a EthindexDB instance
+    def get_ethindex_db_for_exchange(self, address: Optional[str] = None):
+        """return an EthindexDB instance
         This is being used from relay.api to query for events.
         """
-        if self.use_eth_index:
-            return ethindex_db.EthindexDB(
-                ethindex_db.connect(""),
-                address=address,
-                standard_event_types=unw_eth_events.standard_event_types,
-                event_builders=unw_eth_events.event_builders,
-                from_to_types=unw_eth_events.from_to_types,
-            )
-        else:
-            return self.unw_eth_proxies[address]
-
-    def get_event_selector_for_exchange(self, address):
-        """return either a proxy or a EthindexDB instance
-        This is being used from relay.api to query for events.
-        """
-        if self.use_eth_index:
-            return ethindex_db.EthindexDB(
-                ethindex_db.connect(""),
-                address=address,
-                standard_event_types=exchange_events.standard_event_types,
-                event_builders=exchange_events.event_builders,
-                from_to_types=exchange_events.from_to_types,
-            )
-        else:
-            return self.orderbook._exchange_proxies[address]
+        return ethindex_db.ExchangeEthindexDB(
+            ethindex_db.connect(""),
+            address=address,
+            standard_event_types=exchange_events.standard_event_types,
+            event_builders=exchange_events.event_builders,
+            from_to_types=exchange_events.from_to_types,
+        )
 
     def is_currency_network(self, address: str) -> bool:
         return address in self.network_addresses
@@ -292,7 +253,7 @@ class TrustlinesRelay:
     ):
         if not end_time:
             end_time = int(time.time())
-        event_selector = self.get_event_selector_for_currency_network(network_address)
+        event_selector = self.get_ethindex_db_for_currency_network(network_address)
         return EventsInformationFetcher(
             event_selector
         ).get_list_of_paid_interests_for_trustline_in_between_timestamps(
@@ -300,17 +261,19 @@ class TrustlinesRelay:
         )
 
     def get_transfer_information_for_tx_hash(self, tx_hash):
-        fetcher = EventsInformationFetcher(
-            self.get_generic_event_selector_for_currency_networks()
-        )
+        fetcher = EventsInformationFetcher(self.get_ethindex_db_for_currency_network())
         return fetcher.get_transfer_details_for_tx(tx_hash)
 
     def get_transfer_information_from_event_id(self, block_hash, log_index):
-        fetcher = EventsInformationFetcher(
-            self.get_generic_event_selector_for_currency_networks()
-        )
+        fetcher = EventsInformationFetcher(self.get_ethindex_db_for_currency_network())
         return fetcher.get_transfer_details_for_id(block_hash, log_index)
 
+    def get_paid_delegation_fees_for_tx_hash(self, tx_hash):
+        event_proxy = IdentityProxy(self._web3, abi=self.contracts["Identity"]["abi"])
+        return event_proxy.get_transaction_events(
+            tx_hash, event_types=FeePaymentEventType
+        )
+
     def deploy_identity(self, factory_address, implementation_address, signature):
         return self.delegate.deploy_identity(
             factory_address, implementation_address, signature
@@ -330,6 +293,9 @@ class TrustlinesRelay:
             "balance": self.node.balance(identity_address),
             "identity": identity_address,
             "nextNonce": self.delegate.calc_next_nonce(identity_address),
+            "implementationAddress": self.delegate.get_implementation_address(
+                identity_address
+            ),
         }
 
     def _install_w3_middleware(self):
@@ -349,7 +315,7 @@ class TrustlinesRelay:
             )
         except ValueError:
             raise ValueError(
-                f"Could not decrypt keystore. Please make sure the password is correct."
+                "Could not decrypt keystore. Please make sure the password is correct."
             )
 
         account = eth_account.Account.from_key(private_key)
@@ -387,7 +353,7 @@ class TrustlinesRelay:
 
     def start(self):
         self._load_gas_price_settings(self.config["relay"]["gas_price_computation"])
-        self._load_contracts()
+        self.contracts = load_packaged_contracts()
         if self.config["exchange"]["enable"]:
             self._load_orderbook()
         if self.config["push_notification"]["enable"]:
@@ -398,8 +364,7 @@ class TrustlinesRelay:
         self._log_listener = LogFilterListener(self._web3)
         if self.config["delegate"]["enable"]:
             self._start_delegate()
-        # For now always start listening, until we properly have separated the concerns
-        self._start_listen_on_new_addresses()
+        self._load_addresses()
 
     def new_network(self, address: str) -> None:
         assert is_checksum_address(address)
@@ -523,17 +488,14 @@ class TrustlinesRelay:
         type: str = None,
         from_block: int = 0,
     ) -> List[BlockchainEvent]:
-        proxy = self.get_event_selector_for_currency_network(network_address)
+        ethindex_db = self.get_ethindex_db_for_currency_network(network_address)
         if type is not None:
-            events = proxy.get_network_events(
-                type,
-                user_address,
-                from_block=from_block,
-                timeout=self.event_query_timeout,
+            events = ethindex_db.get_network_events(
+                type, user_address, from_block=from_block,
             )
         else:
-            events = proxy.get_all_network_events(
-                user_address, from_block=from_block, timeout=self.event_query_timeout
+            events = ethindex_db.get_all_network_events(
+                user_address, from_block=from_block
             )
         return events
 
@@ -549,19 +511,16 @@ class TrustlinesRelay:
             event_types = None
         else:
             event_types = [type]
-        proxy: Union[ethindex_db.EthindexDB, CurrencyNetworkProxy]
-        if self.use_eth_index:
-            proxy = ethindex_db.EthindexDB(
-                ethindex_db.connect(""),
-                address=network_address,
-                standard_event_types=currency_network_events.trustline_event_types,
-                event_builders=currency_network_events.event_builders,
-                from_to_types=currency_network_events.from_to_types,
-            )
-        else:
-            proxy = self.currency_network_proxies[network_address]
 
-        events = proxy.get_trustline_events(
+        ethindex = ethindex_db.CurrencyNetworkEthindexDB(
+            ethindex_db.connect(""),
+            address=network_address,
+            standard_event_types=currency_network_events.trustline_event_types,
+            event_builders=currency_network_events.event_builders,
+            from_to_types=currency_network_events.from_to_types,
+        )
+
+        events = ethindex.get_trustline_events(
             network_address,
             user_address,
             counterparty_address,
@@ -573,23 +532,15 @@ class TrustlinesRelay:
     def get_network_events(
         self, network_address: str, type: str = None, from_block: int = 0
     ) -> List[BlockchainEvent]:
-        proxy = self.get_event_selector_for_currency_network(network_address)
+        ethindex_db = self.get_ethindex_db_for_currency_network(network_address)
         if type is not None:
-            events = proxy.get_events(
-                type, from_block=from_block, timeout=self.event_query_timeout
-            )
+            events = ethindex_db.get_events(type, from_block=from_block)
         else:
-            events = proxy.get_all_events(
-                from_block=from_block, timeout=self.event_query_timeout
-            )
+            events = ethindex_db.get_all_events(from_block=from_block)
         return events
 
     def get_user_events(
-        self,
-        user_address: str,
-        type: str = None,
-        from_block: int = 0,
-        timeout: float = None,
+        self, user_address: str, type: str = None, from_block: int = 0,
     ) -> List[BlockchainEvent]:
         assert is_checksum_address(user_address)
         event_types: Optional[List[str]]
@@ -616,92 +567,9 @@ class TrustlinesRelay:
             contract_types=contract_types,
         )
         return ethindex.get_all_contract_events(
-            event_types,
-            user_address=user_address,
-            from_block=from_block,
-            timeout=timeout,
+            event_types, user_address=user_address, from_block=from_block,
         )
 
-    def _get_network_event_queries(
-        self, user_address: str, type: str = None, from_block: int = 0
-    ):
-        assert is_checksum_address(user_address)
-        queries = []
-        for network_address in self.network_addresses:
-            currency_network_proxy = self.get_event_selector_for_currency_network(
-                network_address
-            )
-            if type is not None and type in currency_network_proxy.event_types:
-                queries.append(
-                    functools.partial(
-                        currency_network_proxy.get_network_events,
-                        type,
-                        user_address=user_address,
-                        from_block=from_block,
-                    )
-                )
-            else:
-                queries.append(
-                    functools.partial(
-                        currency_network_proxy.get_all_network_events,
-                        user_address=user_address,
-                        from_block=from_block,
-                    )
-                )
-        return queries
-
-    def _get_unw_eth_event_queries(
-        self, user_address: str, type: str = None, from_block: int = 0
-    ):
-        assert is_checksum_address(user_address)
-        queries = []
-        for unw_eth_address in self.unw_eth_addresses:
-            unw_eth_proxy = self.get_event_selector_for_unw_eth(unw_eth_address)
-            if type is not None and type in unw_eth_proxy.event_types:
-                queries.append(
-                    functools.partial(
-                        unw_eth_proxy.get_unw_eth_events,
-                        type,
-                        user_address=user_address,
-                        from_block=from_block,
-                    )
-                )
-            else:
-                queries.append(
-                    functools.partial(
-                        unw_eth_proxy.get_all_unw_eth_events,
-                        user_address=user_address,
-                        from_block=from_block,
-                    )
-                )
-        return queries
-
-    def _get_exchange_event_queries(
-        self, user_address: str, type: str = None, from_block: int = 0
-    ):
-        assert is_checksum_address(user_address)
-        queries = []
-        for exchange_address in self.exchange_addresses:
-            exchange_proxy = self.get_event_selector_for_exchange(exchange_address)
-            if type is not None and type in exchange_proxy.standard_event_types:
-                queries.append(
-                    functools.partial(
-                        exchange_proxy.get_exchange_events,
-                        type,
-                        user_address=user_address,
-                        from_block=from_block,
-                    )
-                )
-            else:
-                queries.append(
-                    functools.partial(
-                        exchange_proxy.get_all_exchange_events,
-                        user_address=user_address,
-                        from_block=from_block,
-                    )
-                )
-        return queries
-
     def get_user_token_events(
         self,
         token_address: str,
@@ -709,21 +577,20 @@ class TrustlinesRelay:
         type: str = None,
         from_block: int = 0,
     ) -> List[BlockchainEvent]:
+
         if token_address in self.unw_eth_addresses:
-            proxy: Union[UnwEthProxy, TokenProxy] = self.get_event_selector_for_unw_eth(
-                token_address
-            )
-            func_names = ["get_unw_eth_events", "get_all_unw_eth_events"]
+            ethindex_db = self.get_ethindex_db_for_unw_eth(token_address)
         else:
-            proxy = self.get_event_selector_for_token(token_address)
-            func_names = ["get_token_events", "get_all_token_events"]
+            ethindex_db = self.get_ethindex_db_for_token(token_address)
 
         if type is not None:
-            events = getattr(proxy, func_names[0])(
-                type, user_address, from_block=from_block
+            events = getattr(ethindex_db, "get_user_events")(
+                event_type=type, user_address=user_address, from_block=from_block
             )
         else:
-            events = getattr(proxy, func_names[1])(user_address, from_block=from_block)
+            events = getattr(ethindex_db, "get_all_contract_events")(
+                user_address=user_address, from_block=from_block
+            )
 
         return events
 
@@ -732,27 +599,25 @@ class TrustlinesRelay:
     ) -> List[BlockchainEvent]:
 
         if token_address in self.unw_eth_addresses:
-            proxy: Union[UnwEthProxy, TokenProxy] = self.get_event_selector_for_unw_eth(
-                token_address
-            )
+            ethindex_db = self.get_ethindex_db_for_unw_eth(token_address)
         else:
-            proxy = self.get_event_selector_for_token(token_address)
+            ethindex_db = self.get_ethindex_db_for_token(token_address)
 
         if type is not None:
-            events = proxy.get_events(type, from_block=from_block)
+            events = ethindex_db.get_events(type, from_block=from_block)
         else:
-            events = proxy.get_all_events(from_block=from_block)
+            events = ethindex_db.get_all_events(from_block=from_block)
 
         return events
 
     def get_exchange_events(
         self, exchange_address: str, type: str = None, from_block: int = 0
     ) -> List[BlockchainEvent]:
-        proxy = self.get_event_selector_for_exchange(exchange_address)
+        ethindex_db = self.get_ethindex_db_for_exchange(exchange_address)
         if type is not None:
-            events = proxy.get_events(type, from_block=from_block)
+            events = ethindex_db.get_events(type, from_block=from_block)
         else:
-            events = proxy.get_all_events(from_block=from_block)
+            events = ethindex_db.get_all_events(from_block=from_block)
         return events
 
     def get_user_exchange_events(
@@ -762,33 +627,29 @@ class TrustlinesRelay:
         type: str = None,
         from_block: int = 0,
     ) -> List[BlockchainEvent]:
-        proxy = self.get_event_selector_for_exchange(exchange_address)
+        ethindex_db = self.get_ethindex_db_for_exchange(exchange_address)
         if type is not None:
-            events = proxy.get_exchange_events(
-                type,
-                user_address,
-                from_block=from_block,
-                timeout=self.event_query_timeout,
+            events = ethindex_db.get_user_events(
+                event_type=type, user_address=user_address, from_block=from_block,
             )
         else:
-            events = proxy.get_all_exchange_events(
-                user_address, from_block=from_block, timeout=self.event_query_timeout
+            events = ethindex_db.get_all_contract_events(
+                user_address=user_address, from_block=from_block,
             )
         return events
 
     def get_all_user_exchange_events(
-        self,
-        user_address: str,
-        type: str = None,
-        from_block: int = 0,
-        timeout: float = None,
+        self, user_address: str, type: str = None, from_block: int = 0,
     ) -> List[BlockchainEvent]:
         assert is_checksum_address(user_address)
-        exchange_event_queries = self._get_exchange_event_queries(
-            user_address, type, from_block
+
+        exchange_ethindex = self.get_ethindex_db_for_exchange()
+        return exchange_ethindex.get_all_exchange_events_of_user(
+            user_address=user_address,
+            all_exchange_addresses=self.exchange_addresses,
+            type=type,
+            from_block=from_block,
         )
-        results = concurrency_utils.joinall(exchange_event_queries, timeout=timeout)
-        return sorted_events(list(itertools.chain.from_iterable(results)))
 
     def _load_gas_price_settings(self, gas_price_settings: Dict):
         method = gas_price_settings["method"]
@@ -807,12 +668,6 @@ class TrustlinesRelay:
                 )
             self.fixed_gas_price = fixed_gas_price
 
-    def _load_contracts(self):
-        with open(
-            os.path.join(sys.prefix, "trustlines-contracts", "build", "contracts.json")
-        ) as data_file:
-            self.contracts = json.load(data_file)
-
     def _load_orderbook(self):
         logger.info("Start exchange orderbook")
         self.orderbook = OrderBookGreenlet()
@@ -873,25 +728,11 @@ class TrustlinesRelay:
             self._process_network_freeze, start_log_filter=False
         )
 
-    def _start_listen_on_new_addresses(self):
-        logger.info("Start trustlines networks index")
-
-        def listen():
-            while True:
-                try:
-                    self._load_addresses()
-                except Exception:
-                    logger.critical(
-                        "Error while loading addresses", exc_info=sys.exc_info()
-                    )
-                sleep(self.config["relay"]["update_indexed_networks_interval"])
-
-        return gevent.Greenlet.spawn(listen)
-
     def _start_push_service(self):
         logger.info("Start pushnotification service")
         path = self.config["push_notification"]["firebase_credentials_path"]
-        self._firebase_raw_push_service = FirebaseRawPushService(path)
+        app = create_firebase_app_from_path_to_keyfile(path)
+        self._firebase_raw_push_service = FirebaseRawPushService(app)
         self._client_token_db = ClientTokenDB(engine=create_engine())
         logger.info("Firebase pushservice started")
         self._start_pushnotifications_for_registered_users()
@@ -935,11 +776,13 @@ class TrustlinesRelay:
         self._publish_blockchain_event(transfer_event)
 
     def _process_trustline_request(self, trustline_request_event):
-        logger.debug("Process trustline request event")
+        logger.debug("Process trustline request event: %s", trustline_request_event)
         self._publish_blockchain_event(trustline_request_event)
 
     def _process_trustline_request_cancel(self, trustline_request_cancel_event):
-        logger.debug("Process trustline request cancel event")
+        logger.debug(
+            "Process trustline request cancel event: %s", trustline_request_cancel_event
+        )
         self._publish_blockchain_event(trustline_request_cancel_event)
 
     def _process_network_freeze(self, network_freeze_event):
@@ -983,7 +826,7 @@ class TrustlinesRelay:
             self._publish_user_event(ev)
 
     def _process_trustline_update(self, trustline_update_event):
-        logger.debug("Process trustline update event")
+        logger.debug("Process trustline update event: %s", trustline_update_event)
         graph = self.currency_network_graphs[trustline_update_event.network_address]
         graph.update_trustline(
             creditor=trustline_update_event.from_,
